-- bug #487319: Pre-Goldmember sql upgrade scripts lack several updates
-- We need to put unique constraint RHN_CLIENTCAPNAM_NAME_UNQ on table
-- rhnClientCapabilityName column NAME.
--
-- In 5.0 -> 5.1 sql upgrade script we did:
-- create unique index rhn_clientcapnam_name_unq
--     on rhnClientCapabilityName(name);
-- 
-- What should have been done:
-- alter table rhnClientCapabilityName
--     add constraint rhn_clientcapnam_name_unq unique(name);
--
-- We cannot do:
-- alter table rhnClientCapabilityName
--     add constraint rhn_clientcapnam_name_unq unique(name)
--     using index rhn_kvt_label_unq;
-- because we need to achieve consistent behavior when it comes to dropping
-- the constraint.

declare
	n user_constraints.constraint_name%TYPE;
begin
	select constraint_name into n from user_constraints
		where constraint_name = 'RHN_CLIENTCAPNAM_NAME_UNQ';
	exception
		when no_data_found then
			execute immediate 'drop index rhn_clientcapnam_name_unq';
			execute immediate 'alter table rhnClientCapabilityName ' || 
				'add constraint rhn_clientcapnam_name_unq unique(name)';
end;
/
