-------------------------------------------------------------------------------
-- Execute this script in a 3.6-214 schema to upgrade it to 3.6-215
-------------------------------------------------------------------------------

set echo on;
whenever sqlerror exit failure;

spool satellite-3.6-214-to-3.6-215.log;

variable evr_id number;
variable epoch varchar2(16);
variable version varchar2(64);
variable release varchar2(64);
variable message varchar2(80);

set define off;
set sqlblanklines on;

declare
   cursor evrs is
      select   e.id, e.epoch, e.version, e.release, e.evr
      from  rhnPackageEVR e,
         rhnVersionInfo rvi
      where rvi.label = 'schema'
         and rvi.name_id =
            lookup_package_name('rhn-satellite-schema')
         and rvi.evr_id = e.id;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','214')
         );
begin
   :evr_id := null;
   :message := 'XXX Invalid satellite schema version.';
   for evr in evrs loop
      :evr_id := evr.id;
      :epoch := evr.epoch;
      :version := evr.version;
      :release := evr.release;
      :message :=  '*** Schema version is currently ' ||
         evr.evr.as_vre_simple() ||
         ', and will NOT be upgraded';
      for vevr in valid_evrs loop
         :message :=  '*** Schema version is currently ' ||
            evr.evr.as_vre_simple() ||
            ', and will be upgraded';
      end loop;
      return;
   end loop;
end;
/

select :message from dual;

declare
   invalid_schema_version exception;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','214')
         );
begin
   for vevr in valid_evrs loop
      return;
   end loop;
   raise invalid_schema_version;
end;
/
show errors

--
-- DROP TRIGGER rhn_errata_mod_trig
--
drop trigger rhn_errata_mod_trig;

create or replace trigger
rhn_errata_mod_trig
before insert or update on rhnErrata
for each row
declare
	date_to_use	date;
	rc_modified	date;
	cursor channels is
		select	channel_id id
		from	rhnChannelErrata
		where	errata_id = :new.id;
begin
        date_to_use := sysdate;
        if :new.last_modified = :old.last_modified then
            :new.last_modified := date_to_use;
        end if;
	if inserting then
		return;
	end if;
	if (:old.id != :new.id) or
	  (:old.advisory != :new.advisory) or
	  (:old.advisory_type != :new.advisory_type) or
	  (:old.advisory_name != :new.advisory_name) or
	  (:old.advisory_rel != :new.advisory_rel) or
	  (:old.product != :new.product) or
	  (:old.description != :new.description) or
	  (:old.synopsis != :new.synopsis) or
	  (:old.topic != :new.topic) or
	  (:old.solution != :new.solution) or
	  (:old.issue_date != :new.issue_date) or
	  (:old.update_date != :new.update_date) or
	  (:old.refers_to != :new.refers_to) then
	  	:new.modified := date_to_use;
	end if;
	for channel in channels loop
		-- lock the table.
		select	last_modified
		into	rc_modified
		from	rhnChannel
		where	id = channel.id
		for update;
	
		if date_to_use <= rc_modified then
			-- alter session set 
			--   nls_date_format ='DD-MON-YYYY HH24:MI:SS';
			-- select sysdate, sysdate + .00001 , sysdate from dual;
			-- select sysdate, sysdate + 1/86400, sysdate from dual;
			date_to_use := rc_modified + 1/86400;
		end if;
		update rhnChannel
			set last_modified = date_to_use
			where id = channel.id;
	end loop;
end rhn_errata_mod_trig;
/
show errors

--
-- Update Schema version
--
update rhnVersionInfo set evr_id = lookup_evr(null, '3.6', '215')
   where label = 'schema'
   and name_id = lookup_package_name('rhn-satellite-schema');
commit;

select   '*** Schema version is now ' || e.evr.as_vre_simple()
from  rhnPackageEVR e, rhnVersionInfo vi
where vi.evr_id = e.id
  and   vi.label = 'schema';

spool off;
exit;

