
set echo on;
whenever sqlerror exit failure;

spool satellite-3.6-215-to-3.6-216.log;

variable evr_id number;
variable epoch varchar2(16);
variable version varchar2(64);
variable release varchar2(64);
variable message varchar2(80);

set define off;
set sqlblanklines on;

declare
   cursor evrs is
      select   e.id, e.epoch, e.version, e.release, e.evr
      from  rhnPackageEVR e,
         rhnVersionInfo rvi
      where rvi.label = 'schema'
         and rvi.name_id =
            lookup_package_name('rhn-satellite-schema')
         and rvi.evr_id = e.id;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','215')
         );
begin
   :evr_id := null;
   :message := 'XXX Invalid satellite schema version.';
   for evr in evrs loop
      :evr_id := evr.id;
      :epoch := evr.epoch;
      :version := evr.version;
      :release := evr.release;
      :message :=  '*** Schema version is currently ' ||
         evr.evr.as_vre_simple() ||
         ', and will NOT be upgraded';
      for vevr in valid_evrs loop
         :message :=  '*** Schema version is currently ' ||
            evr.evr.as_vre_simple() ||
            ', and will be upgraded';
      end loop;
      return;
   end loop;
end;
/

select :message from dual;


declare
   invalid_schema_version exception;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','215')
         );
begin
   for vevr in valid_evrs loop
      return;
   end loop;
   raise invalid_schema_version;
end;
/
show errors

create index rhn_ser_act_sid_aid_s_idx
        on rhnServerAction(server_id, action_id, status)
        storage( freelists 16 )
        initrans 32;

alter table rhnServerAction
modify constraint rhn_server_action_sid_aid_uq
using index rhn_ser_act_sid_aid_s_idx;

drop index rhn_server_action_sid_aid_idx;


update rhnVersionInfo set evr_id = lookup_evr(null, '3.6', '216')
   where label = 'schema'
   and name_id = lookup_package_name('rhn-satellite-schema');
commit;

select   '*** Schema version is now ' || e.evr.as_vre_simple()
from  rhnPackageEVR e, rhnVersionInfo vi
where vi.evr_id = e.id
  and   vi.label = 'schema';

spool off;
exit;

